/*********************************************
Titre : Calcul d'autonnomie restante
Nom : Gaillard ; Vaslin
Postcondition : Affichage de l'autonnomie restante
Date : 01/2004
Nom du fichier : autonomie


Chip type           : AT90S8535
Clock frequency     : 4,000000 MHz
Memory model        : Small
Internal SRAM size  : 512
External SRAM size  : 0
Data Stack size     : 128
*********************************************/

#include <90s8535.h>
#include <ssi.h>
#include <stdlib.h>
#include <stdio.h>
#include <delay.h>

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x15
#endasm
#include <lcd.h>

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input;
// Start the AD conversion
ADCSR|=0x40;
// Wait for the AD conversion to complete
while ((ADCSR & 0x10)==0);
ADCSR|=0x10;
return ADCW;
}

// Declare your global variables here
char display_buffer_ligne_0[17];
char display_buffer_ligne_1[17];

void main(void)
{
//Variables             
/*
Vr : Volume reservoir
Ht : hauteur total reservoir
He : hauteur essence
V : volume d'essence restant
C : consommation moyenne
A : autonomie
Km : nb kilometre
*/
int N, N_1, K, K_1;
float Vr=50.0,Ht=1023.0,He,V,Km=200.0,C,A; 
//Vr=0.05m^3
//Ht=255m
//Km=200km
char V_aff[5],C_aff[5],A_aff[5],K_aff[5];

// Input/Output Ports initialization
// Port A initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=In Func6=In Func7=In 
// State0=P State1=P State2=P State3=P State4=P State5=P State6=T State7=T 
PORTA=0x3F;
DDRA=0x00;


// Port C initialization
// Func0=In Func1=In Func2=In Func3=In Func4=In Func5=In Func6=In Func7=In 
// State0=T State1=T State2=T State3=T State4=T State5=T State6=T State7=T 
PORTC=0x00;
DDRC=0x00;

// ADC initialization
// ADC Clock frequency: 31,250 kHz
ADCSR=0x87;

// LCD module initialization
lcd_init(16);

while (1)
      { 
        delay_ms(500);		// Mesure toutes les 0.5s
        K = read_adc(7);  	// Kilomtrage parcouru
        Km = K;
        itoa(K,K_aff);
        N = read_adc(6);  	// Mesure du niveau d'essence
        He=N;   	  
	V=(He*Vr)/Ht;     	// Calcul du volume d'essence restant
	ftoa(V,1,V_aff);
	C=(100*(Vr-V))/Km;      // Calcul de la consommation moyenne
	ftoa(C,1,C_aff);
	A=(100*V)/C;            // Calcul de l'autonomie restante
       	ftoa(A,0,A_aff);
         
        
        if ((N != N_1)|| (K != K_1))           // Affichage sur un LCD 2 lignes de 16 caractres
 	{ 
          sprintf(display_buffer_ligne_0,"Cb=%sL D=%sKm",V_aff,K_aff);
          sprintf(display_buffer_ligne_1,"Co=%sL A=%sKm",C_aff,A_aff);
	  Affiche_LCD(display_buffer_ligne_0,display_buffer_ligne_1);
	  N_1=N; K_1=K;
	} 
      };
}
